#ifdef _WIN32
#include "windows.h"
#else
#include "pthread.h"
#endif

#include "NxCoreAPI_Wrapper_C.h"

#ifdef _WIN32
__declspec(thread) bool g_successfulExclude = false;
#else
_Thread_local bool g_successfulExclude = false;
#endif

char* pszFilename;

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg) {
	if (pNxCoreMsg->MessageType == NxMSG_STATUS && !g_successfulExclude) {
		int optionPermissions[4] = { 2, 20, 21, 22 };
		for (int i = 0; i < 4; i++) {
			int permission = optionPermissions[i];
			if (pNxCoreSys->UserData != permission && pfExcludeOpraExch(NxPERMID_REMOVE | permission) != 0)
				return NxCALLBACKRETURN_CONTINUE;
		}
		g_successfulExclude = true;
	}

	if (pNxCoreMsg->MessageType == NxMSG_TRADE) {
		const NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
		NxOptionHdr* pnxOptionHdr = &pHeader->pnxOptionHdr;
		if (pnxOptionHdr != 0) {
			char* symbol = pHeader->pnxStringSymbol->String;
			const NxTime* pTimestamp = &pHeader->nxExgTimestamp;
			const NxDate* pExpireDate = &pnxOptionHdr->nxExpirationDate;
			double strikePrice = pfNxCorePriceToDouble(pnxOptionHdr->strikePrice, 7);
			char* contractType;
			if (pnxOptionHdr->PutCall == 1)
				contractType = "Put";
			else
				contractType = "Call";

			const NxCoreTrade* pTrade = &pNxCoreMsg->coreData.Trade;
			double price = pfNxCorePriceToDouble(pTrade->Price, pTrade->PriceType);
			int size = pTrade->Size;

			printf("Trade for %s %s expiring %d%02d%02d with strike $%.2f at %02d:%02d:%02d for %d shares at $%.2f\n",
				symbol, contractType, pExpireDate->Year, pExpireDate->Month, pExpireDate->Day, strikePrice,
				pTimestamp->Hour, pTimestamp->Minute, pTimestamp->Second, size,price);
		}
	}
	return NxCALLBACKRETURN_CONTINUE;
}
void thProcess(size_t permission) {
	int returnValue = pfNxCoreProcessTape(pszFilename, NULL, NxCF_EXCLUDE_CRC_CHECK, permission, OnNxCoreCallback);
	processReturnValue(returnValue);
}
int main(int argc, char* argv[]) {
	if (argc < 3)
		return 1;
	if (!LoadNxCore(argv[1])){
		printf("loading library failed\n");
		return 2;
	}
	pszFilename = argv[2];

#ifdef _WIN32
	HANDLE threads[4];
	threads[0] = CreateThread(NULL, 0, thProcess, 2, 0, 0);
	threads[1] = CreateThread(NULL, 0, thProcess, 20, 0, 0);
	threads[2] = CreateThread(NULL, 0, thProcess, 21, 0, 0);
	threads[3] = CreateThread(NULL, 0, thProcess, 22, 0, 0);

	WaitForMultipleObjects(4, threads, TRUE, INFINITE);

	for (int i = 0; i<4; i++)
		CloseHandle(threads[i]);
#else
	pthread_t threads[4];
	pthread_create(&threads[0],0,thProcess,2);
	pthread_create(&threads[1],0,thProcess,20);
	pthread_create(&threads[2],0,thProcess,21);
	pthread_create(&threads[3],0,thProcess,22);

	for (int i = 0; i<4; i++)
		pthread_join(threads[i],NULL);
#endif

	return 0;
}
