#include "stdio.h"
#include "NxCoreAPI_Wrapper_C.h"

int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg) {
	if (pNxCoreMsg->MessageType == NxMSG_MMQUOTE) {

		const NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
		char* symbol = pHeader->pnxStringSymbol->String;
		const NxTime* pTimestamp = &pHeader->nxExgTimestamp;

		const NxCoreMMQuote* pQuote = &pNxCoreMsg->coreData.MMQuote;
		const NxCoreQuote* pCoreQuote = &pQuote->coreQuote;
		char* marketMaker = pQuote->pnxStringMarketMaker->String;
		int sizeBid = pCoreQuote->BidSize;
		int sizeAsk = pCoreQuote->AskSize;
		double priceBid = pfNxCorePriceToDouble(pCoreQuote->BidPrice, pCoreQuote->PriceType);
		double priceAsk = pfNxCorePriceToDouble(pCoreQuote->AskPrice, pCoreQuote->PriceType);

		printf("Market Maker Quote for %s at %02d:%02d:%02d by: %s Bid: %d lots at $%.2f Ask: %d lots at $%.2f\n",
			symbol, pTimestamp->Hour, pTimestamp->Minute, pTimestamp->Second,
			marketMaker, sizeBid, priceBid, sizeAsk, priceAsk);
	}
	if(pNxCoreMsg->MessageType == NxMSG_EXGQUOTE) {
		
		const NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
		char* symbol = pHeader->pnxStringSymbol->String;
		const NxTime* pTimestamp = &pHeader->nxExgTimestamp;
		
		const NxCoreExgQuote* pQuote = &pNxCoreMsg->coreData.ExgQuote;
		if (!pQuote->BBOChangeFlags)
			return NxCALLBACKRETURN_CONTINUE;
		int sizeBid = pQuote->BestBidSize;
		int sizeAsk = pQuote->BestAskSize;
		double priceBid = pfNxCorePriceToDouble(pQuote->BestBidPrice, pQuote->coreQuote.PriceType);
		double priceAsk = pfNxCorePriceToDouble(pQuote->BestAskPrice, pQuote->coreQuote.PriceType);
		
		printf("New NBBO Quote for %s at %02d:%02d:%02d Bid: %d lots at $%.2f Ask: %d lots at $%.2f\n",
			symbol, pTimestamp->Hour, pTimestamp->Minute, pTimestamp->Second,
			sizeBid, priceBid, sizeAsk, priceAsk);
	}
	return NxCALLBACKRETURN_CONTINUE;  
}

int main(int argc, char* argv[]) {
	if (argc < 3)
		return 1;

	if (LoadNxCore(argv[1])){
		int returnValue = pfNxCoreProcessTape(argv[2], 0, 0, 0, OnNxCoreCallback);
		processReturnValue(returnValue);
	}
	else
		printf("loading library failed\n");

	return 0;
}