#include "stdio.h"
#include "NxCoreAPI_Wrapper_C.h"


int OnNxCoreCallback(const NxCoreSystem* pNxCoreSys, const NxCoreMessage* pNxCoreMsg) {
	if(pNxCoreMsg->MessageType == NxMSG_TRADE) {
		
		const NxCoreHeader* pHeader = &pNxCoreMsg->coreHeader;
		char* symbol = pHeader->pnxStringSymbol->String;
		const NxTime* pTs = &pHeader->nxExgTimestamp;
		
		const char* exchangeLong = pfNxCoreGetDefinedString(NxST_EXCHANGE, pHeader->ReportingExg);
		char exchange[64];
		char *quotPtr = strchr(exchangeLong, '|');
		if (quotPtr) {
			memcpy(exchange, exchangeLong, quotPtr - exchangeLong);
			exchange[quotPtr - exchangeLong] = 0;
		}
		else
			strcpy(exchange, exchangeLong);
		
		const NxCoreTrade* pTrade = &pNxCoreMsg->coreData.Trade;
		double price = pfNxCorePriceToDouble(pTrade->Price, pTrade->PriceType);
		int size = pTrade->Size;
		unsigned int sequence = pTrade->ExgSequence;

		char conditions[64] = "";
		char* p = conditions;
		for (int i = 0; i < 4; i++) {
			char condition = pTrade->ExtTradeConditions[i];
			const char* conditionName = pfNxCoreGetDefinedString(NxST_TRADECONDITION, condition);
			p += sprintf(p,"%s|", (conditionName ? conditionName :""));
		}

		char eligibility[64] = "";
		p = eligibility;
		if (pTrade->ConditionFlags) {
			p += sprintf(p,"(Doesn't set");
			if (pTrade->ConditionFlags & NxTCF_NOHIGH)
				p += sprintf(p, " high");
			if (pTrade->ConditionFlags & NxTCF_NOLOW)
				p += sprintf(p, " low");
			if (pTrade->ConditionFlags & NxTCF_NOLAST)
				p += sprintf(p, " last");
			p += sprintf(p, ")");
		}

		if(pTrade->PriceFlags & NxTPF_EXGCANCEL)
			printf("Cancel for %s at sequence %u on %s %d shares at $%.02f\n",
				symbol, sequence, exchange, size, price);
		else
			printf("Trade for %s at %02d:%02d:%02d sequence %u on %s for %d shares at $%.02f %s%s\n", 
				symbol, pTs->Hour, pTs->Minute, pTs->Second, sequence, exchange, size, price, conditions, eligibility);
	}
	return NxCALLBACKRETURN_CONTINUE;  
}

int main(int argc, char* argv[]) {
	if (argc < 3)
		return 1;

	if (LoadNxCore(argv[1])){
		int returnValue = pfNxCoreProcessTape(argv[2], 0, 0, 0, OnNxCoreCallback);
		processReturnValue(returnValue);
	}
	else
		printf("loading library failed\n");

	return 0;
}
